--<< ====================================================================== >>--
-- Class Setup                                                                --
--<< ====================================================================== >>--
ScoreSort = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDebug-2.0", "AceHook-2.1")

function ScoreSort:OnInitialize()
	local cmd = { 
		type = "group",
		args = {
			report = {
				name = "Report", type = "execute",
				desc = "Report the current sorting parameters.",
				func = function() self:Print("Sorting by", self.sortby, "in", self.order.."ending", "order.") end,
			},
		}
	}

	self:RegisterChatCommand({"/score","/ss", "/scs"}, cmd)
end

function ScoreSort:OnEnable()
	self.sortby = SCORE_KILLING_BLOWS -- default sort value
	self.order = "desc" -- ascending / descending
	
	self:Hook("WorldStateScoreFrame_Update", true)
	
	getglobal("WorldStateScoreFrame"):SetScript("OnShow", function() ScoreSort:ScoreFrameOnShow() end)
	getglobal("WorldStateScoreFrameKB"):SetScript("OnMouseUp",	   function() ScoreSort:OnColumnClick("WorldStateScoreFrameKB") end)
	getglobal("WorldStateScoreFrameDeaths"):SetScript("OnMouseUp",     function() ScoreSort:OnColumnClick("WorldStateScoreFrameDeaths") end)
	getglobal("WorldStateScoreFrameHK"):SetScript("OnMouseUp",	   function() ScoreSort:OnColumnClick("WorldStateScoreFrameHK") end)
	getglobal("WorldStateScoreFrameDamageDone"):SetScript("OnMouseUp", function() ScoreSort:OnColumnClick("WorldStateScoreFrameDamageDone") end)
	getglobal("WorldStateScoreFrameHealingDone"):SetScript("OnMouseUp",function() ScoreSort:OnColumnClick("WorldStateScoreFrameHealingDone") end)
	getglobal("WorldStateScoreFrameHonorGained"):SetScript("OnMouseUp",function() ScoreSort:OnColumnClick("WorldStateScoreFrameHonorGained") end)
	for i=1, 7 do
		getglobal("WorldStateScoreColumn"..i):SetScript("OnMouseUp",function() ScoreSort:OnColumnClick("WorldStateScoreColumn"..i) end)
	end
end

function ScoreSort:OnDisable()
	getglobal("WorldStateScoreFrame"):SetScript("OnShow", nil)
	getglobal("WorldStateScoreFrameKB"):SetScript("OnMouseUp", nil)
	getglobal("WorldStateScoreFrameDeaths"):SetScript("OnMouseUp", nil)
	getglobal("WorldStateScoreFrameHK"):SetScript("OnMouseUp", nil)
	getglobal("WorldStateScoreFrameDamageDone"):SetScript("OnMouseUp", nil)
	getglobal("WorldStateScoreFrameHealingDone"):SetScript("OnMouseUp", nil)
	getglobal("WorldStateScoreFrameHonorGained"):SetScript("OnMouseUp", nil)
	for i=1, 7 do
		getglobal("WorldStateScoreColumn"..i):SetScript("OnMouseUp", nil)
	end

end

--<< ====================================================================== >>--
-- Indexing                                                                   --
--<< ====================================================================== >>--

-- @desc: Gathers the data of each player and builds a sorted index to define the layout of the scoreboard
function ScoreSort:BuildPlayerIndex()
	self.playerIndex = {}

	for i=1, GetNumBattlefieldScores() do
		local name, killingBlows, honorKills, deaths, honorGained, _, _, _, _, damageDone, healingDone = GetBattlefieldScore(i)
		
		if self.sortby == SCORE_KILLING_BLOWS then
			self.playerIndex[i] = {pindex = i, value = killingBlows}
		elseif self.sortby == DEATHS then
			self.playerIndex[i] = {pindex = i, value = deaths}
		elseif self.sortby == SCORE_HONORABLE_KILLS then
			self.playerIndex[i] = {pindex = i, value = honorKills}
		elseif self.sortby == SCORE_DAMAGE_DONE then
			self.playerIndex[i] = {pindex = i, value = damageDone}
		elseif self.sortby == SCORE_HEALING_DONE then
			self.playerIndex[i] = {pindex = i, value = healingDone}
		elseif self.sortby == SCORE_HONOR_GAINED then
			self.playerIndex[i] = {pindex = i, value = honorGained}
		else
			for j=1, GetNumBattlefieldStats() do
				if self.sortby == GetBattlefieldStatInfo(j) then
					local stat = GetBattlefieldStatData(i, j)
					self.playerIndex[i] = {pindex = i, value = stat}
				else
					self.playerIndex[i] = {pindex = i, value = killingBlows} 
				end
			end
		end
	end
	
	if self.order == "desc" then
		table.sort(self.playerIndex, function(a,b) return a.value < b.value end)
	elseif self.order == "asc" then
		table.sort(self.playerIndex, function(a,b) return a.value > b.value end)
	end
end

--<< ====================================================================== >>--
-- Hooks	                                                              --
--<< ====================================================================== >>--

-- @desc: Fired when the scoreboard is shown.
function ScoreSort:ScoreFrameOnShow()
	WorldStateScoreFrame_Resize()
	self:BuildPlayerIndex()
	WorldStateScoreFrame_Update()
	WorldStateScoreFrameTab_OnClick(WorldStateScoreFrameTab1)
end
-- @args: 
--	name - the frame name of the clicked column
-- @desc: Fired when a column title is clicked. Sets sort order and the value to sort by. Then builds an the index.
function ScoreSort:OnColumnClick(name)
	local sortby = getglobal(name.."Text"):GetText()
	
	if sortby == self.sortby then
		if self.order == "desc" then
			self.order = "asc"
		else
			self.order = "desc"
		end
	else
		self.sortby = sortby
		self.order = "desc"
	end
	
	self:BuildPlayerIndex()
	self:WorldStateScoreFrame_Update()
	--self:Print("Sorting by", sortby, "in", self.order, "order.")
end

-- @desc: Updates the scoreboard.
function ScoreSort:WorldStateScoreFrame_Update()

	local isArena, isRegistered = IsActiveBattlefieldArena()

	if ( isArena ) then
		-- Hide unused tabs
		WorldStateScoreFrameTab1:Hide()
		WorldStateScoreFrameTab2:Hide()
		WorldStateScoreFrameTab3:Hide()

		-- Hide unused columns
		WorldStateScoreFrameTeam:Hide()
		WorldStateScoreFrameDeaths:Hide()
		WorldStateScoreFrameHK:Hide()
		WorldStateScoreFrameHonorGained:Hide()

		-- Reanchor some columns.
		WorldStateScoreFrameDamageDone:SetPoint("CENTER", "WorldStateScoreFrameKB", "CENTER", 58, 0)

		if ( isRegistered ) then
			WorldStateScoreFrameTeam:Show()
			WorldStateScoreFrameHonorGainedText:SetText(SCORE_RATING_CHANGE)
			WorldStateScoreFrameHonorGained:Show()
			WorldStateScoreFrameKB:SetPoint("CENTER", "WorldStateScoreFrameTeam", "RIGHT", 120, 0)
		else
			WorldStateScoreFrameKB:SetPoint("CENTER", "WorldStateScoreFrameName", "RIGHT", 160, 0)
		end
	else
		-- Show Tabs
		WorldStateScoreFrameTab1:Show()
		WorldStateScoreFrameTab2:Show()
		WorldStateScoreFrameTab3:Show()

		WorldStateScoreFrameTeam:Hide()
		WorldStateScoreFrameDeaths:Show()
		WorldStateScoreFrameHK:Show()

		WorldStateScoreFrameHKText:SetText(SCORE_HONORABLE_KILLS)

		WorldStateScoreFrameHonorGained:Show()

		-- Reanchor some columns.
		WorldStateScoreFrameDamageDone:SetPoint("CENTER", "WorldStateScoreFrameHK", "CENTER", 58, 0)
		WorldStateScoreFrameKB:SetPoint("CENTER", "WorldStateScoreFrameName", "RIGHT", 160, 0)
	end

	--Show the frame if its hidden and there is a victor
	if ( GetBattlefieldWinner() ) then

		-- Show the final score frame, set textures etc.
		ShowUIPanel(WorldStateScoreFrame)

		if ( isArena ) then
			WorldStateScoreFrameLeaveButton:SetText(LEAVE_ARENA)
			WorldStateScoreFrameTimerLabel:SetText(TIME_TO_PORT_ARENA)
		else
			WorldStateScoreFrameLeaveButton:SetText(LEAVE_BATTLEGROUND)
			WorldStateScoreFrameTimerLabel:SetText(TIME_TO_PORT)
		end

		WorldStateScoreFrameLeaveButton:Show()
		WorldStateScoreFrameTimerLabel:Show()
		WorldStateScoreFrameTimer:Show()

		-- Show winner
		local battlefieldWinner = GetBattlefieldWinner()

		if ( isArena ) then
			if ( isRegistered ) then
				local teamName, teamRating, newTeamRating = GetBattlefieldTeamInfo(battlefieldWinner)
				WorldStateScoreWinnerFrameText:SetText(format(VICTORY_TEXT_ARENA_WINS, teamName))
			else
				WorldStateScoreWinnerFrameText:SetText(getglobal("VICTORY_TEXT_ARENA"..battlefieldWinner))
			end

			if ( battlefieldWinner == 0 ) then
				-- Green Team won
				WorldStateScoreWinnerFrameLeft:SetVertexColor(0.19, 0.57, 0.11)
				WorldStateScoreWinnerFrameRight:SetVertexColor(0.19, 0.57, 0.11)
				WorldStateScoreWinnerFrameText:SetVertexColor(0.1, 1.0, 0.1)
			else
				-- Gold Team won
				WorldStateScoreWinnerFrameLeft:SetVertexColor(0.85, 0.71, 0.26)
				WorldStateScoreWinnerFrameRight:SetVertexColor(0.85, 0.71, 0.26)
				WorldStateScoreWinnerFrameText:SetVertexColor(1, 0.82, 0)
			end
		else
			WorldStateScoreWinnerFrameText:SetText(getglobal("VICTORY_TEXT"..battlefieldWinner))

			if ( battlefieldWinner == 0 ) then
				-- Horde won
				WorldStateScoreWinnerFrameLeft:SetVertexColor(0.52, 0.075, 0.18)
				WorldStateScoreWinnerFrameRight:SetVertexColor(0.5, 0.075, 0.18)
				WorldStateScoreWinnerFrameText:SetVertexColor(1.0, 0.1, 0.1)
			else
				-- Alliance won
				WorldStateScoreWinnerFrameLeft:SetVertexColor(0.11, 0.26, 0.51)
				WorldStateScoreWinnerFrameRight:SetVertexColor(0.11, 0.26, 0.51)
				WorldStateScoreWinnerFrameText:SetVertexColor(0, 0.68, 0.94)
			end
		end

		WorldStateScoreWinnerFrame:Show()

	else
		WorldStateScoreWinnerFrame:Hide()
		WorldStateScoreFrameLeaveButton:Hide()
		WorldStateScoreFrameTimerLabel:Hide()
		WorldStateScoreFrameTimer:Hide()
	end


	-- Update buttons
	local numScores = GetNumBattlefieldScores()

	local scoreButton, buttonIcon, buttonName, nameButton, buttonKills, buttonDeaths, buttonHonorGained, buttonFaction, columnButtonText, columnButtonIcon, buttonFactionLeft, buttonFactionRight, buttonDamage, buttonHealing, buttonTeam
	local name, kills, killingBlows, deaths, honorGained, faction, rank, race, class, damageDone, healingDone
	local index
	local columnData

	-- ScrollFrame update
	local hasScrollBar
	if ( numScores > MAX_WORLDSTATE_SCORE_BUTTONS ) then
		hasScrollBar = 1
		WorldStateScoreScrollFrame:Show()
	else
		WorldStateScoreScrollFrame:Hide()
	end

	FauxScrollFrame_Update(WorldStateScoreScrollFrame, numScores, MAX_WORLDSTATE_SCORE_BUTTONS, 16 )

	-- Setup Columns
	local text, icon, tooltip, columnButton
	local numStatColumns = GetNumBattlefieldStats()
	local columnButton, columnButtonText, columnTextButton, columnIcon
	local honorGainedAnchorFrame = "WorldStateScoreFrameHealingDone"

	for i=1, MAX_NUM_STAT_COLUMNS do

		if ( i <= numStatColumns ) then
			text, icon, tooltip = GetBattlefieldStatInfo(i)
			columnButton = getglobal("WorldStateScoreColumn"..i)
			columnButtonText = getglobal("WorldStateScoreColumn"..i.."Text")
			columnButtonText:SetText(text)
			columnButton.icon = icon
			columnButton.tooltip = tooltip
			
			columnTextButton = getglobal("WorldStateScoreButton1Column"..i.."Text")

			if ( icon ~= "" ) then
				columnTextButton:SetPoint("CENTER", "WorldStateScoreColumn"..i, "CENTER", 6, -33)
			else
				columnTextButton:SetPoint("CENTER", "WorldStateScoreColumn"..i, "CENTER", -1, -33)
			end
	
			if ( i == numStatColumns ) then
				honorGainedAnchorFrame = "WorldStateScoreColumn"..i
			end

			getglobal("WorldStateScoreColumn"..i):Show()
		else
			getglobal("WorldStateScoreColumn"..i):Hide()
		end
	end

	-- Anchor the bonus honor gained to the last column shown
	WorldStateScoreFrameHonorGained:SetPoint("CENTER", honorGainedAnchorFrame, "CENTER", 58, 0)

	-- Last button shown is what the player count anchors to
	local lastButtonShown = "WorldStateScoreButton1"

	for i=1, MAX_WORLDSTATE_SCORE_BUTTONS do

		-- Need to create an index adjusted by the scrollframe offset
		index = FauxScrollFrame_GetOffset(WorldStateScoreScrollFrame) + i

		scoreButton = getglobal("WorldStateScoreButton"..i)

		if ( hasScrollBar ) then
			scoreButton:SetWidth(WorldStateScoreFrame.scrollBarButtonWidth)
		else
			scoreButton:SetWidth(WorldStateScoreFrame.buttonWidth)
		end

		if ( index <= numScores ) then
			buttonIcon = getglobal("WorldStateScoreButton"..i.."RankButtonIcon")
			buttonName = getglobal("WorldStateScoreButton"..i.."NameButtonName")
			nameButton = getglobal("WorldStateScoreButton"..i.."NameButton")
			buttonTeam = getglobal("WorldStateScoreButton"..i.."Team")
			buttonKills = getglobal("WorldStateScoreButton"..i.."HonorableKills")
			buttonKillingBlows = getglobal("WorldStateScoreButton"..i.."KillingBlows")
			buttonDeaths = getglobal("WorldStateScoreButton"..i.."Deaths")
			buttonDamage = getglobal("WorldStateScoreButton"..i.."Damage")
			buttonHealing = getglobal("WorldStateScoreButton"..i.."Healing")
			buttonHonorGained = getglobal("WorldStateScoreButton"..i.."HonorGained")
			buttonFactionLeft = getglobal("WorldStateScoreButton"..i.."FactionLeft")
			buttonFactionRight = getglobal("WorldStateScoreButton"..i.."FactionRight")
			
			local playerSlot 
			if self.playerIndex[index] then 
				playerSlot = self.playerIndex[index].pindex 
			else 
				playerSlot = index 
			end

			name, killingBlows, honorableKills, deaths, honorGained, faction, rank, race, class, damageDone, healingDone = GetBattlefieldScore(playerSlot)
			rankName, rankNumber = GetPVPRankInfo(rank, faction)

			if ( rankNumber > 0 ) then
				buttonIcon:SetTexture(format("%s%02d","Interface\\PvPRankBadges\\PvPRank", rankNumber))
				buttonIcon:Show()
			else
				buttonIcon:Hide()
			end

			buttonName:SetText(name)
			nameButton:SetWidth(buttonName:GetWidth())

			if ( not race ) then
				race = ""
			end

			if ( not class ) then
				class = ""
			end

			nameButton.name = name
			nameButton.tooltip = race.." "..class

			getglobal("WorldStateScoreButton"..i.."RankButton").tooltip = rankName

			buttonKillingBlows:SetText(killingBlows)

			buttonDamage:SetText(damageDone)

			buttonHealing:SetText(healingDone)

			 

			local teamName, teamRating, newTeamRating = GetBattlefieldTeamInfo(faction)

			if ( isArena ) then
				if ( isRegistered ) then
					buttonTeam:SetText(teamName)
					buttonTeam:Show()
					buttonHonorGained:SetText(tostring(newTeamRating - teamRating))
					buttonHonorGained:Show()
				else
					buttonHonorGained:Hide()
					buttonTeam:Hide()
				end
				buttonKills:Hide()
				buttonDeaths:Hide()
			else
				buttonKills:SetText(honorableKills)
				buttonDeaths:SetText(deaths)
				buttonHonorGained:SetText(honorGained)
				buttonTeam:Hide()
				buttonKills:Show()
				buttonDeaths:Show()
				buttonHonorGained:Show()
			end

			for j=1, MAX_NUM_STAT_COLUMNS do
				columnButtonText = getglobal("WorldStateScoreButton"..i.."Column"..j.."Text")
				columnButtonIcon = getglobal("WorldStateScoreButton"..i.."Column"..j.."Icon")

				if ( j <= numStatColumns ) then
				-- If there's an icon then move the icon left and format the text with an "x" in front
				columnData = GetBattlefieldStatData(index, j)
					
					if ( getglobal("WorldStateScoreColumn"..j).icon ~= "" ) then
						if ( columnData > 0 ) then
							columnButtonText:SetText(format(FLAG_COUNT_TEMPLATE, columnData))
							columnButtonIcon:SetTexture(getglobal("WorldStateScoreColumn"..j).icon..faction)
							columnButtonIcon:Show()
						else
							columnButtonText:SetText("")
							columnButtonIcon:Hide()
						end
					else
						columnButtonText:SetText(columnData)
						columnButtonIcon:Hide()
					end
					columnButtonText:Show()
				else
					columnButtonText:Hide()
					columnButtonIcon:Hide()
				end
			end

			if ( faction ) then
				if ( faction == 0 ) then
					if ( isArena ) then

						-- Green Team
						buttonFactionLeft:SetVertexColor(0.19, 0.57, 0.11)
						buttonFactionRight:SetVertexColor(0.19, 0.57, 0.11)
						buttonName:SetVertexColor(0.1, 1.0, 0.1)
					else
						-- Horde
						buttonFactionLeft:SetVertexColor(0.52, 0.075, 0.18)
						buttonFactionRight:SetVertexColor(0.5, 0.075, 0.18)
						buttonName:SetVertexColor(1.0, 0.1, 0.1)
					end
				else	
					if ( isArena ) then
						-- Gold Team
						buttonFactionLeft:SetVertexColor(0.85, 0.71, 0.26)
						buttonFactionRight:SetVertexColor(0.85, 0.71, 0.26)
						buttonName:SetVertexColor(1, 0.82, 0)
					else
						-- Alliance
						buttonFactionLeft:SetVertexColor(0.11, 0.26, 0.51)
						buttonFactionRight:SetVertexColor(0.11, 0.26, 0.51)
						buttonName:SetVertexColor(0, 0.68, 0.94)
					end
				end

				if ( ( not isArena ) and ( name == UnitName("player") ) ) then
					buttonName:SetVertexColor(1.0, 0.82, 0)
				end

				buttonFactionLeft:Show()
				buttonFactionRight:Show()
			else
				buttonFactionLeft:Hide()
				buttonFactionRight:Hide()
			end

			lastButtonShown = scoreButton:GetName()

			scoreButton:Show()
		else
			scoreButton:Hide()
		end
	end

	 

	-- Count number of players on each side
	local numHorde = 0
	local numAlliance = 0

	for i=1, numScores do
		name, killingBlows, honorableKills, deaths, honorGained, faction, rank, race, class = GetBattlefieldScore(i)

		if ( faction ) then
			if ( faction == 0 ) then
				numHorde = numHorde + 1
			else
				numAlliance = numAlliance + 1
			end
		end
	end

	-- Set count text and anchor team count to last button shown
	WorldStateScorePlayerCount:Show()

	if ( numHorde > 0 and numAlliance > 0 ) then
		WorldStateScorePlayerCount:SetText(format(GetText("PLAYER_COUNT_ALLIANCE", nil, numAlliance), numAlliance).." / "..format(GetText("PLAYER_COUNT_HORDE", nil, numHorde), numHorde))
	elseif ( numAlliance > 0 ) then
		WorldStateScorePlayerCount:SetText(format(GetText("PLAYER_COUNT_ALLIANCE", nil, numAlliance), numAlliance))
	elseif ( numHorde > 0 ) then
		WorldStateScorePlayerCount:SetText(format(GetText("PLAYER_COUNT_HORDE", nil, numHorde), numHorde))
	else
		WorldStateScorePlayerCount:Hide()
	end

	if ( isArena ) then
		WorldStateScorePlayerCount:Hide()
	end

	WorldStateScorePlayerCount:SetPoint("TOPLEFT", lastButtonShown, "BOTTOMLEFT", 15, -6)
	WorldStateScoreBattlegroundRunTime:SetText(TIME_ELAPSED.." "..SecondsToTime(GetBattlefieldInstanceRunTime()/1000, 1))
	WorldStateScoreBattlegroundRunTime:SetPoint("TOPRIGHT", lastButtonShown, "BOTTOMRIGHT", -20, -7)
end
